SystemOrganization addCategory: #PromisePipelining!
SystemOrganization addCategory: #'PromisePipelining-Tests'!
SystemOrganization addCategory: #'PromisePipelining-Visualisation'!
SystemOrganization addCategory: #'PromisePipelining-Signals'!

Promise subclass: #PPPromise
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PromisePipelining'!

PPPromise class
	instanceVariableNames: 'signal'!

PPPromise class
	instanceVariableNames: 'signal'!

!PPPromise class methodsFor: 'as yet unclassified' stamp: 'WS 1/5/2012 10:09'!
creationSignal
	^ signal ifNil: [ signal := PPSignal new ]! !

!PPPromise class methodsFor: 'as yet unclassified' stamp: 'WS 1/5/2012 10:11'!
disconnectCreation: id
	self creationSignal disconnect: id! !

!PPPromise class methodsFor: 'as yet unclassified' stamp: 'WS 1/5/2012 10:11'!
new
	| promise |
	promise := super new.
	self creationSignal emitWith: promise.
	^ promise! !

!PPPromise class methodsFor: 'as yet unclassified' stamp: 'WS 1/5/2012 10:11'!
onCreation: aBlock
	^ self creationSignal connect: aBlock! !

!PPPromise methodsFor: 'as yet unclassified' stamp: 'WS 12/13/2011 22:05'!
directValue
	^ super value! !

!PPPromise methodsFor: 'as yet unclassified' stamp: 'WS 12/13/2011 22:08'!
doesNotUnderstand: aMessage
	| promise |
	promise := PPPromise new.
	self whenResolved: [ promise resolveWith: (aMessage sendTo: self directValue) ].
	^ promise! !

!PPPromise methodsFor: 'as yet unclassified' stamp: 'WS 1/5/2012 12:17'!
initialize
	super initialize.
	signal := PPSignal new! !

!PPPromise methodsFor: 'as yet unclassified' stamp: 'WS 1/5/2012 12:14'!
resolveWith: arg
	"Resolve this promise"
	mutex critical: [
		isResolved ifTrue: [ ^ self ].
		value := arg.
		isResolved := true.
		resolvers ifNotNil: [resolvers do: [:r | self evaluateResolver: r]].
	]! !

!PPPromise methodsFor: 'as yet unclassified' stamp: 'WS 12/13/2011 22:03'!
value
	^ self wait! !

TestCase subclass: #PPPromiseTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PromisePipelining-Tests'!

!PPPromiseTest methodsFor: 'as yet unclassified' stamp: 'WS 12/14/2011 11:15'!
testDirectValue
	| promise result |
	promise := PPPromise new.
	result := Object new.
	
	self assert: [ promise directValue isNil ].
	
	promise resolveWith: result.
	
	self assert: [ promise directValue == result ].! !

!PPPromiseTest methodsFor: 'as yet unclassified' stamp: 'WS 12/14/2011 11:34'!
testPipelining
	| promise pipelinedPromise |
	promise := PPPromise new.
	pipelinedPromise := promise + 5.
	
	promise resolveWith: 5.
	self assert: [ pipelinedPromise value = 10 ].! !

!PPPromiseTest methodsFor: 'as yet unclassified' stamp: 'WS 1/5/2012 12:16'!
testResolvingThrowsNoException
	| promise resolveValue |
	promise := PPPromise new.
	resolveValue := 5.
	promise resolveWith: resolveValue.
	self assert: promise value = resolveValue.
	promise resolveWith: 2.
	self assert: promise value = resolveValue! !

!PPPromiseTest methodsFor: 'as yet unclassified' stamp: 'WS 12/14/2011 11:30'!
testValue
	| promise hasWaited result process innerComparison innerHasWaited |
	promise := PPPromise new.
	hasWaited := false.
	result := Object new.
	
	process := [
		innerComparison := promise value == result.
		innerHasWaited := hasWaited.
	] fork.
	hasWaited := true.
	
	promise resolveWith: result.
	
	[ process isTerminated ] whileFalse.

	self assert: innerComparison.
	self assert: innerHasWaited.! !

Object subclass: #PPPromisePipeliningVisualisation
	instanceVariableNames: 'window codeWidget executeButton listWidget canvasWidget resolveButton resolveText promises selectedPromise'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PromisePipelining-Visualisation'!

!PPPromisePipeliningVisualisation class methodsFor: 'as yet unclassified' stamp: 'WS 12/20/2011 21:56'!
open
	^ self new
		open;
		yourself! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/5/2012 11:01'!
accept: aString
	^ false! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/5/2012 11:43'!
buttonState
	^ true! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/5/2012 16:43'!
codeText
	^ ''! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/5/2012 10:55'!
defaultExtent
	^ 1200 @ 800! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/5/2012 16:38'!
executeButtonText
	^ 'Execute'! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/5/2012 16:42'!
executeCode
	| signalId |
	signalId := PPPromise onCreation: [ :promise | self promiseCreated: promise ].
	self initializePromises.
	listWidget updateList.
	codeWidget textMorph editor evaluateSelection.
	PPPromise disconnectCreation: signalId! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/5/2012 16:46'!
initialize
	| scrollPane |
	super initialize.
	self initializePromises.
	
	codeWidget := PluggableTextMorphPlus
		on: self
		text: #codeText
		accept: #accept:.
	codeWidget styler: (SHTextStylerST80 new view: codeWidget; yourself).
	codeWidget wantsFrameAdornments: false.
	codeWidget textMorph editor model: self.
	
	executeButton := PluggableButtonMorphPlus on: self getState: #buttonState action: #executeCode label: #executeButtonText.
	
	listWidget := PluggableListMorph on: self list: #promises selected: #selectedPromise changeSelected: #selectedPromise:.
	
	scrollPane := ScrollPane new.
	canvasWidget := Morph new.
	scrollPane scroller addMorph: canvasWidget.
	
	resolveButton := PluggableButtonMorphPlus on: self getState: #buttonState action: #resolveSelectedPromise label: #resolveButtonText.
	resolveText := PluggableTextMorphPlus on: self text: #resolveText accept: #accept:.
	resolveText styler: (SHTextStylerST80 new view: resolveText; yourself).
	resolveText wantsFrameAdornments: false.
	resolveText textMorph editor model: self.
	
	window := SystemWindow new.
	window setWindowColor: window defaultColor.
	window setLabel: self labelString.
	window addMorph: codeWidget frame: (0 @ 0 corner: 0.7 @ 0.3).
	window addMorph: executeButton frame: (0.7 @ 0 corner: 1 @ 0.05).
	window addMorph: listWidget frame: (0.7 @ 0.05 corner: 1 @ 0.75).
	"window addMorph: scrollPane frame: (0 @ 0.3 corner: 0.75 @ 1)."
	window addMorph: resolveButton frame: (0.7 @ 0.75 corner: 1 @ 0.8).
	window addMorph: resolveText frame: (0.7 @ 0.8 corner: 1 @ 1).
	window model: self.! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/5/2012 12:28'!
initializePromises
	promises := OrderedCollection new.
	selectedPromise := 0! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 12/20/2011 22:02'!
labelString
	^ 'Promise Pipelining Visualisation'! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/5/2012 16:41'!
open
	window openInWorldExtent: self defaultExtent! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/5/2012 12:27'!
promiseCreated: promise
	promises add: promise.
	promise whenResolved: [ self resolved: promise ].
	listWidget updateList.! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/5/2012 10:57'!
promises
	^ promises! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/5/2012 16:44'!
resolveButtonText
	^ 'Resolve With'! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/5/2012 16:50'!
resolveSelectedPromise
	self resolveSelectedPromiseWith: (resolveText textMorph editor evaluateSelection)! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/5/2012 15:24'!
resolveSelectedPromiseWith: anObject
	| promise |
	promise := promises at: selectedPromise ifAbsent: [ ^ false ].
	
	promise resolveWith: anObject.
	resolveText update: #resolveText.! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/5/2012 11:44'!
resolveText
	^ ''! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/5/2012 16:33'!
resolved: promise
	listWidget updateList! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/5/2012 10:57'!
selectedPromise
	^ selectedPromise! !

!PPPromisePipeliningVisualisation methodsFor: 'as yet unclassified' stamp: 'WS 1/5/2012 11:01'!
selectedPromise: newIndex
	selectedPromise := newIndex.
	listWidget updateList! !

Object subclass: #PPSignal
	instanceVariableNames: 'callbacks nextId'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PromisePipelining-Signals'!

!PPSignal methodsFor: 'as yet unclassified' stamp: 'RL 12/14/2011 13:45'!
connect: aBlock
	| id |
	id := nextId.
	callbacks at: id put: aBlock.
	nextId := nextId + 1.
	^ id
	! !

!PPSignal methodsFor: 'as yet unclassified' stamp: 'RL 12/14/2011 13:45'!
disconnect: anId
	^ callbacks removeKey: anId ifAbsent: []! !

!PPSignal methodsFor: 'as yet unclassified' stamp: 'RL 12/16/2011 20:40'!
disconnectAll
	^ callbacks removeAll! !

!PPSignal methodsFor: 'as yet unclassified' stamp: 'RL 12/14/2011 13:34'!
emit
	self emitWithArguments: #()! !

!PPSignal methodsFor: 'as yet unclassified' stamp: 'RL 12/14/2011 14:36'!
emitWith: arg1
	self emitWithArguments: (Array with: arg1)! !

!PPSignal methodsFor: 'as yet unclassified' stamp: 'RL 12/14/2011 14:36'!
emitWith: arg1 with: arg2
	self emitWithArguments: (Array with: arg1 with: arg2)! !

!PPSignal methodsFor: 'as yet unclassified' stamp: 'RL 12/14/2011 14:36'!
emitWith: arg1 with: arg2 with: arg3
	self emitWithArguments: (Array with: arg1 with: arg2 with: arg3)! !

!PPSignal methodsFor: 'as yet unclassified' stamp: 'RL 12/14/2011 13:57'!
emitWithArguments: arguments
	callbacks do: [ :callback | 
		callback valueWithArguments: (self resizeArguments: arguments toFit: callback argumentCount)
	]! !

!PPSignal methodsFor: 'as yet unclassified' stamp: 'RL 12/14/2011 14:28'!
initialize
	super initialize.
	callbacks := Dictionary new.
	nextId := 0.! !

!PPSignal methodsFor: 'as yet unclassified' stamp: 'RL 12/14/2011 14:40'!
resizeArguments: arguments toFit: aSize
	| newArgs |
	newArgs := Array new: aSize.
	1 to: aSize do: [ :i |
		newArgs at: i put: (arguments at: i ifAbsent: [ nil ])
	].
	^ newArgs! !

Object subclass: #PPSignalHandler
	instanceVariableNames: 'signals'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PromisePipelining-Signals'!

!PPSignalHandler methodsFor: 'as yet unclassified' stamp: 'WS 1/4/2012 13:41'!
connect: signalName to: aBlock
	^ ( signals at: signalName ifAbsentPut: [ PPSignal new ])
		connect: aBlock! !

!PPSignalHandler methodsFor: 'as yet unclassified' stamp: 'RL 12/14/2011 13:46'!
disconnect: signalName from: anId
	(signals at: signalName ifAbsent: [ ^ self ]) disconnect: anId! !

!PPSignalHandler methodsFor: 'as yet unclassified' stamp: 'RL 12/14/2011 13:47'!
emit: signalName
	 (signals at: signalName ifAbsentPut: [ ^ self ]) emit! !

!PPSignalHandler methodsFor: 'as yet unclassified' stamp: 'RL 12/16/2011 20:05'!
emit: signalName with: arg1
	 (signals at: signalName ifAbsentPut: [ ^ self ])
		emitWithArguments: (Array with: arg1)! !

!PPSignalHandler methodsFor: 'as yet unclassified' stamp: 'RL 12/16/2011 20:05'!
emit: signalName with: arg1 with: arg2
	 (signals at: signalName ifAbsentPut: [ ^ self ])
		emitWithArguments: (Array with: arg1 with: arg2)! !

!PPSignalHandler methodsFor: 'as yet unclassified' stamp: 'RL 12/16/2011 20:05'!
emit: signalName with: arg1 with: arg2 with: arg3
	 (signals at: signalName ifAbsentPut: [ ^ self ])
		emitWithArguments: (Array with: arg1 with: arg2 with: arg3)! !

!PPSignalHandler methodsFor: 'as yet unclassified' stamp: 'RL 12/14/2011 13:47'!
emit: signalName withArguments: arguments
	 (signals at: signalName ifAbsentPut: [ ^ self ]) emitWithArguments: arguments! !

!PPSignalHandler methodsFor: 'as yet unclassified' stamp: 'RL 12/14/2011 13:36'!
initialize
	super initialize.
	signals := Dictionary new! !
